@@ -1,3 +1,4 @@
+from itertools import permutations
 import re
 import pytest
 
@@ -52,7 +53,6 @@ class TestWKTElement():
         assert unpickled.srid == 3
         assert unpickled.extended is True
         assert unpickled.data == 'POINT(1 2)'
-        assert unpickled.name == 'ST_GeomFromEWKT'
         f = unpickled.ST_Buffer(2)
         eq_sql(f, 'ST_Buffer('
                'ST_GeomFromEWKT(:ST_GeomFromEWKT_1), '
@@ -62,6 +62,11 @@ class TestWKTElement():
             u'ST_GeomFromEWKT_1': 'POINT(1 2)',
         }
 
+    def test_eq(self):
+        a = WKTElement('POINT(1 2)')
+        b = WKTElement('POINT(1 2)')
+        assert a == b
+
 
 class TestExtendedWKTElement():
 
@@ -92,7 +97,6 @@ class TestExtendedWKTElement():
         assert unpickled.srid == self._srid
         assert unpickled.extended is True
         assert unpickled.data == self._ewkt
-        assert unpickled.name == 'ST_GeomFromEWKT'
         f = unpickled.ST_Buffer(2)
         eq_sql(f, 'ST_Buffer('
                'ST_GeomFromEWKT(:ST_GeomFromEWKT_1), '
@@ -120,6 +124,11 @@ class TestExtendedWKTElement():
         with pytest.raises(ArgumentError):
             WKTElement('SRID=BAD SRID;POINT (1 2 3)', extended=True)
 
+    def test_eq(self):
+        a = WKTElement(self._ewkt, extended=True)
+        b = WKTElement(self._ewkt, extended=True)
+        assert a == b
+
 
 class TestWKTElementFunction():
 
@@ -215,7 +224,6 @@ class TestExtendedWKBElement():
         assert unpickled.srid == self._srid
         assert unpickled.extended is True
         assert unpickled.data == bytes_(self._bin)
-        assert unpickled.name == 'ST_GeomFromEWKB'
         f = unpickled.ST_Buffer(2)
         eq_sql(f, 'ST_Buffer('
                'ST_GeomFromEWKB(:ST_GeomFromEWKB_1), '
@@ -244,6 +252,11 @@ class TestExtendedWKBElement():
         assert e.srid == self._srid
         assert wkb.loads(e.data, hex=True).wkt == self._wkt
 
+    def test_eq(self):
+        a = WKBElement(self._bin, extended=True)
+        b = WKBElement(self._bin, extended=True)
+        assert a == b
+
 
 class TestWKBElement():
 
@@ -270,18 +283,94 @@ class TestWKBElement():
         e = WKBElement(b'\x01\x02')
         assert isinstance(str(e), str)
 
+    def test_eq(self):
+        a = WKBElement(b'\x01\x02')
+        b = WKBElement(b'\x01\x02')
+        assert a == b
+
+
+class TestNotEqualSpatialElement():
+
+    # _bin/_hex computed by following query:
+    # SELECT ST_GeomFromEWKT('SRID=3;POINT(1 2)');
+    _ewkb = buffer_(b'\x01\x01\x00\x00 \x03\x00\x00\x00\x00\x00\x00'
+                    b'\x00\x00\x00\xf0?\x00\x00\x00\x00\x00\x00\x00@')
+    _wkb = wkb.loads(bytes_(_ewkb)).wkb
+    _hex = str_('010100002003000000000000000000f03f0000000000000040')
+    _srid = 3
+    _wkt = 'POINT (1 2)'
+    _ewkt = 'SRID=3;POINT (1 2)'
+
+    def test_eq(self):
+        a = WKBElement(self._ewkb, extended=True)
+        b = WKBElement(self._wkb, srid=self._srid)
+        c = WKTElement(self._wkt, srid=self._srid)
+        d = WKTElement(self._ewkt, extended=True)
+        e = WKBElement(self._hex, extended=True)
+        assert a == a
+        assert b == b
+        assert c == c
+        assert d == d
+        assert e == e
+        assert a == e and e == a
+
+    def test_neq_other_types(self):
+        a = WKBElement(self._ewkb, extended=True)
+        b = WKBElement(self._wkb, srid=self._srid)
+        c = WKTElement(self._wkt, srid=self._srid)
+        d = WKTElement(self._ewkt, extended=True)
+        e = WKBElement(self._hex, extended=True)
+        all_elements = [a, b, c, d, None, 1, "test"]
+        for i, j in permutations(all_elements, 2):
+            assert i != j
+        for i in all_elements[1:]:
+            assert i != e and e != i
+
 
 class TestRasterElement():
 
+    rast_data = (
+        b'\x01\x00\x00\x01\x00\x9a\x99\x99\x99\x99\x99\xc9?\x9a\x99\x99\x99\x99\x99'
+        b'\xc9\xbf\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xf0?\x00'
+        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xe6\x10\x00'
+        b'\x00\x05\x00\x05\x00D\x00\x01\x01\x01\x01\x01\x01\x01\x01\x01\x00\x01\x01'
+        b'\x01\x00\x00\x01\x01\x00\x00\x00\x01\x00\x00\x00\x00')
+
+    hex_rast_data = (
+        '01000001009a9999999999c93f9a9999999999c9bf0000000000000000000000000000f03'
+        'f00000000000000000000000000000000e610000005000500440001010101010101010100'
+        '010101000001010000000100000000')
+
     def test_desc(self):
-        e = RasterElement(b'\x01\x02')
-        assert e.desc == '0102'
+        e = RasterElement(self.rast_data)
+        assert e.desc == self.hex_rast_data
+        assert e.srid == 4326
+        e = RasterElement(self.hex_rast_data)
+        assert e.desc == self.hex_rast_data
+        assert e.srid == 4326
 
     def test_function_call(self):
-        e = RasterElement(b'\x01\x02')
+        e = RasterElement(self.rast_data)
         f = e.ST_Height()
-        eq_sql(f, 'ST_Height(:raster_1::raster)')
-        assert f.compile().params == {u'raster_1': b'\x01\x02'}
+        eq_sql(f, 'ST_Height(raster(:raster_1))')
+        assert f.compile().params == {u'raster_1': self.hex_rast_data}
+
+    def test_pickle_unpickle(self):
+        import pickle
+        e = RasterElement(self.rast_data)
+        assert e.srid == 4326
+        assert e.extended is True
+        assert e.data == self.hex_rast_data
+        pickled = pickle.dumps(e)
+        unpickled = pickle.loads(pickled)
+        assert unpickled.srid == 4326
+        assert unpickled.extended is True
+        assert unpickled.data == self.hex_rast_data
+        f = unpickled.ST_Height()
+        eq_sql(f, 'ST_Height(raster(:raster_1))')
+        assert f.compile().params == {
+            u'raster_1': self.hex_rast_data,
+        }
 
 
 class TestCompositeElement():
