@@ -1,3 +1,5 @@
+from json import loads
+import os
 from pkg_resources import parse_version
 import pytest
 
@@ -10,11 +12,12 @@ else:
     del compat
 
 from sqlalchemy import __version__ as SA_VERSION
-from sqlalchemy import create_engine, Table, MetaData, Column, Integer, bindparam
+from sqlalchemy import create_engine
+from sqlalchemy import Table, MetaData, Column, Integer, String, bindparam
 from sqlalchemy.orm import sessionmaker
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.engine import reflection
-from sqlalchemy.exc import DataError, IntegrityError, InternalError
+from sqlalchemy.exc import DataError, IntegrityError, InternalError, ProgrammingError
 from sqlalchemy.sql import select, func
 from sqlalchemy.sql.expression import type_coerce
 
@@ -22,10 +25,13 @@ from geoalchemy2 import Geometry, Geography, Raster
 from geoalchemy2.elements import WKTElement, WKBElement, RasterElement
 from geoalchemy2.shape import from_shape
 
-from shapely.geometry import LineString
+from shapely.geometry import LineString, Point
+
+from . import skip_postgis1, skip_postgis2
 
 
-engine = create_engine('postgresql://gis:gis@localhost/gis', echo=True)
+engine = create_engine(
+    os.environ.get('PYTEST_DB_URL', 'postgresql://gis:gis@localhost/gis'), echo=False)
 metadata = MetaData(engine)
 Base = declarative_base(metadata=metadata)
 
@@ -44,6 +50,7 @@ class Poi(Base):
     __tablename__ = 'poi'
     __table_args__ = {'schema': 'gis'}
     id = Column(Integer, primary_key=True)
+    geom = Column(Geometry(geometry_type='POINT', srid=4326))
     geog = Column(Geography(geometry_type='POINT', srid=4326))
 
     def __init__(self, geog):
@@ -78,7 +85,7 @@ class IndexTestWithoutSchema(Base):
 
 session = sessionmaker(bind=engine)()
 
-postgis_version = session.execute(func.postgis_version()).scalar()
+postgis_version = session.execute(func.postgis_lib_version()).scalar()
 if postgis_version.startswith('1.'):
     # With PostGIS 1.x the AddGeometryColumn and DropGeometryColumn
     # management functions should be used.
@@ -97,11 +104,6 @@ else:
         def __init__(self, rast):
             self.rast = rast
 
-postgis2_required = pytest.mark.skipif(
-    postgis_version.startswith('1.'),
-    reason="requires PostGIS 2 or higher",
-)
-
 
 class TestIndex():
 
@@ -212,6 +214,49 @@ class TestInsertionCore():
         srid = session.execute(row[1].ST_SRID()).scalar()
         assert srid == 4326
 
+    def test_insert_geom_poi(self):
+        conn = self.conn
+
+        conn.execute(Poi.__table__.insert(), [
+            {'geom': 'SRID=4326;POINT(1 1)'},
+            {'geom': WKTElement('POINT(1 1)', srid=4326)},
+            {'geom': WKTElement('SRID=4326;POINT(1 1)', extended=True)},
+            {'geom': from_shape(Point(1, 1), srid=4326)},
+            {'geom': from_shape(Point(1, 1), srid=4326, extended=True)}
+        ])
+
+        results = conn.execute(Poi.__table__.select())
+        rows = results.fetchall()
+
+        for row in rows:
+            assert isinstance(row[1], WKBElement)
+            wkt = session.execute(row[1].ST_AsText()).scalar()
+            assert wkt == 'POINT(1 1)'
+            srid = session.execute(row[1].ST_SRID()).scalar()
+            assert srid == 4326
+            assert row[1] == from_shape(Point(1, 1), srid=4326, extended=True)
+
+    def test_insert_geog_poi(self):
+        conn = self.conn
+
+        conn.execute(Poi.__table__.insert(), [
+            {'geog': 'SRID=4326;POINT(1 1)'},
+            {'geog': WKTElement('POINT(1 1)', srid=4326)},
+            {'geog': WKTElement('SRID=4326;POINT(1 1)', extended=True)},
+            {'geog': from_shape(Point(1, 1), srid=4326)}
+        ])
+
+        results = conn.execute(Poi.__table__.select())
+        rows = results.fetchall()
+
+        for row in rows:
+            assert isinstance(row[2], WKBElement)
+            wkt = session.execute(row[2].ST_AsText()).scalar()
+            assert wkt == 'POINT(1 1)'
+            srid = session.execute(row[2].ST_SRID()).scalar()
+            assert srid == 4326
+            assert row[2] == from_shape(Point(1, 1), srid=4326)
+
 
 class TestSelectBindParam():
 
@@ -317,7 +362,7 @@ class TestInsertionORM():
         srid = session.execute(lake.geom.ST_SRID()).scalar()
         assert srid == 4326
 
-    @postgis2_required
+    @skip_postgis1(postgis_version)
     def test_Raster(self):
         polygon = WKTElement('POLYGON((0 0,1 1,0 1,0 0))', srid=4326)
         o = Ocean(polygon.ST_AsRaster(5, 5))
@@ -346,6 +391,156 @@ class TestInsertionORM():
         assert bottom_right is None
 
 
+class TestUpdateORM():
+
+    def setup(self):
+        metadata.drop_all(checkfirst=True)
+        metadata.create_all()
+
+    def teardown(self):
+        session.rollback()
+        metadata.drop_all()
+
+    def test_WKTElement(self):
+        raw_wkt = 'LINESTRING(0 0,1 1)'
+        lake = Lake(WKTElement(raw_wkt, srid=4326))
+        session.add(lake)
+
+        # Insert in DB
+        session.flush()
+
+        # Check what was inserted in DB
+        assert isinstance(lake.geom, WKTElement)
+        wkt = session.execute(lake.geom.ST_AsText()).scalar()
+        assert wkt == raw_wkt
+        srid = session.execute(lake.geom.ST_SRID()).scalar()
+        assert srid == 4326
+
+        # Set geometry to None
+        lake.geom = None
+
+        # Update in DB
+        session.flush()
+
+        # Check what was updated in DB
+        assert lake.geom is None
+        assert session.execute(select([Lake])).fetchall() == [(1, None)]
+
+        # Reset geometry to initial value
+        lake.geom = WKTElement(raw_wkt, srid=4326)
+
+        # Update in DB
+        session.flush()
+
+        # Check what was inserted in DB
+        assert isinstance(lake.geom, WKTElement)
+        wkt = session.execute(lake.geom.ST_AsText()).scalar()
+        assert wkt == raw_wkt
+        srid = session.execute(lake.geom.ST_SRID()).scalar()
+        assert srid == 4326
+
+    def test_WKBElement(self):
+        shape = LineString([[0, 0], [1, 1]])
+        lake = Lake(from_shape(shape, srid=4326))
+        session.add(lake)
+
+        # Insert in DB
+        session.flush()
+
+        # Check what was inserted in DB
+        assert isinstance(lake.geom, WKBElement)
+        wkt = session.execute(lake.geom.ST_AsText()).scalar()
+        assert wkt == 'LINESTRING(0 0,1 1)'
+        srid = session.execute(lake.geom.ST_SRID()).scalar()
+        assert srid == 4326
+
+        # Set geometry to None
+        lake.geom = None
+
+        # Update in DB
+        session.flush()
+
+        # Check what was updated in DB
+        assert lake.geom is None
+        assert session.execute(select([Lake])).fetchall() == [(1, None)]
+
+        # Reset geometry to initial value
+        lake.geom = from_shape(shape, srid=4326)
+
+        # Insert in DB
+        session.flush()
+
+        # Check what was inserted in DB
+        assert isinstance(lake.geom, WKBElement)
+        wkt = session.execute(lake.geom.ST_AsText()).scalar()
+        assert wkt == 'LINESTRING(0 0,1 1)'
+        srid = session.execute(lake.geom.ST_SRID()).scalar()
+        assert srid == 4326
+
+    def test_other_type_fail(self):
+        shape = LineString([[0, 0], [1, 1]])
+        lake = Lake(from_shape(shape, srid=4326))
+        session.add(lake)
+
+        # Insert in DB
+        session.flush()
+
+        # Set geometry to 1, which is of wrong type
+        lake.geom = 1
+
+        # Update in DB
+        with pytest.raises(ProgrammingError):
+            # Call __eq__() operator of _SpatialElement with 'other' argument equal to 1
+            session.flush()
+
+    @skip_postgis1(postgis_version)
+    def test_Raster(self):
+        polygon = WKTElement('POLYGON((0 0,1 1,0 1,0 0))', srid=4326)
+        o = Ocean(polygon.ST_AsRaster(5, 5))
+        session.add(o)
+        session.flush()
+        session.expire(o)
+
+        assert isinstance(o.rast, RasterElement)
+
+        rast_data = (
+            '01000001009A9999999999C93F9A9999999999C9BF0000000000000000000000000000F03'
+            'F00000000000000000000000000000000E610000005000500440001010101010101010100'
+            '010101000001010000000100000000'
+        )
+
+        assert o.rast.data == rast_data
+
+        assert session.execute(
+            select([Ocean.rast.ST_Height(), Ocean.rast.ST_Width()])
+        ).fetchall() == [(5, 5)]
+
+        # Set rast to None
+        o.rast = None
+
+        # Insert in DB
+        session.flush()
+        session.expire(o)
+
+        # Check what was updated in DB
+        assert o.rast is None
+        assert session.execute(select([Ocean])).fetchall() == [(1, None)]
+
+        # Reset rast to initial value
+        o.rast = RasterElement(rast_data)
+
+        # Insert in DB
+        session.flush()
+        session.expire(o)
+
+        # Check what was updated in DB
+        assert o.rast.data == rast_data
+
+        assert session.execute(
+            select([Ocean.rast.ST_Height(), Ocean.rast.ST_Width()])
+        ).fetchall() == [(5, 5)]
+
+
 class TestPickle():
 
     def setup(self):
@@ -360,6 +555,7 @@ class TestPickle():
         lake = Lake(WKTElement('LINESTRING(0 0,1 1)', srid=4326))
         session.add(lake)
         session.flush()
+        session.expire(lake)
         return lake.id
 
     def test_pickle_unpickle(self):
@@ -376,7 +572,6 @@ class TestPickle():
         assert unpickled.geom.srid == 4326
         assert str(unpickled.geom) == data_desc
         assert unpickled.geom.extended is True
-        assert unpickled.geom.name == 'ST_GeomFromEWKB'
 
 
 class TestCallFunction():
@@ -393,12 +588,14 @@ class TestCallFunction():
         lake = Lake(WKTElement('LINESTRING(0 0,1 1)', srid=4326))
         session.add(lake)
         session.flush()
+        session.expire(lake)
         return lake.id
 
     def _create_one_poi(self):
         p = Poi('POINT(5 45)')
         session.add(p)
         session.flush()
+        session.expire(p)
         return p.id
 
     def test_ST_GeometryType(self):
@@ -428,6 +625,7 @@ class TestCallFunction():
         assert isinstance(r1, WKBElement)
 
         lake = session.query(Lake).get(lake_id)
+        assert isinstance(lake.geom, WKBElement)
         r2 = session.execute(lake.geom.ST_Buffer(2)).scalar()
         assert isinstance(r2, WKBElement)
 
@@ -445,6 +643,7 @@ class TestCallFunction():
     def test_ST_Dump(self):
         lake_id = self._create_one_lake()
         lake = session.query(Lake).get(lake_id)
+        assert isinstance(lake.geom, WKBElement)
 
         s = select([func.ST_Dump(Lake.__table__.c.geom)])
         r1 = session.execute(s).scalar()
@@ -477,6 +676,7 @@ class TestCallFunction():
     def test_ST_DumpPoints(self):
         lake_id = self._create_one_lake()
         lake = session.query(Lake).get(lake_id)
+        assert isinstance(lake.geom, WKBElement)
 
         dump = lake.geom.ST_DumpPoints()
 
@@ -513,6 +713,64 @@ class TestCallFunction():
                 type_coerce('POINT(5 45)', Geography)) < 1000).one()
         assert poi.id == poi_id
 
+    def test_ST_AsGeoJson(self):
+        lake_id = self._create_one_lake()
+        lake = session.query(Lake).get(lake_id)
+
+        # Test geometry
+        s1 = select([func.ST_AsGeoJSON(Lake.__table__.c.geom)])
+        r1 = session.execute(s1).scalar()
+        assert loads(r1) == {
+            "type": "LineString",
+            "coordinates": [[0, 0], [1, 1]]
+        }
+
+        # Test geometry ORM
+        s1_orm = lake.geom.ST_AsGeoJSON()
+        r1_orm = session.execute(s1_orm).scalar()
+        assert loads(r1_orm) == {
+            "type": "LineString",
+            "coordinates": [[0, 0], [1, 1]]
+        }
+
+        # Test with function inside
+        s1_func = select([func.ST_AsGeoJSON(func.ST_MakeValid(Lake.__table__.c.geom))])
+        r1_func = session.execute(s1_func).scalar()
+        assert loads(r1_func) == {
+            "type": "LineString",
+            "coordinates": [[0, 0], [1, 1]]
+        }
+
+    @skip_postgis1(postgis_version)
+    @skip_postgis2(postgis_version)
+    def test_ST_AsGeoJson_feature(self):
+        self._create_one_lake()
+
+        # Test feature
+        s2 = select([func.ST_AsGeoJSON(Lake, 'geom')])
+        r2 = session.execute(s2).scalar()
+        assert loads(r2) == {
+            "type": "Feature",
+            "geometry": {
+                "type": "LineString",
+                "coordinates": [[0, 0], [1, 1]]
+            },
+            "properties": {"id": 1}
+        }
+
+        # Test feature with subquery
+        ss3 = select([Lake, bindparam('dummy_val', 10).label('dummy_attr')]).alias()
+        s3 = select([func.ST_AsGeoJSON(ss3, 'geom')])
+        r3 = session.execute(s3).scalar()
+        assert loads(r3) == {
+            "type": "Feature",
+            "geometry": {
+                "type": "LineString",
+                "coordinates": [[0, 0], [1, 1]]
+            },
+            "properties": {"dummy_attr": 10, "id": 1}
+        }
+
     @pytest.mark.skipif(
         parse_version(SA_VERSION) < parse_version("1.3.4"),
         reason='Case-insensitivity is only available for sqlalchemy>=1.3.4')
@@ -524,6 +782,7 @@ class TestCallFunction():
         assert isinstance(r1, WKBElement)
 
         lake = session.query(Lake).get(lake_id)
+        assert isinstance(lake.geom, WKBElement)
 
         r2 = session.execute(lake.geom.ST_Buffer(2)).scalar()
         assert isinstance(r2, WKBElement)
@@ -547,6 +806,24 @@ class TestCallFunction():
             r1.data == r2.data == r3.data == r4.data == r5.data == r6.data
             == r7.data)
 
+    def test_unknown_function_column(self):
+        self._create_one_lake()
+
+        s = select([func.ST_UnknownFunction(Lake.__table__.c.geom, 2)])
+        with pytest.raises(ProgrammingError):
+            session.execute(s)
+
+    def test_unknown_function_element(self):
+        lake_id = self._create_one_lake()
+        lake = session.query(Lake).get(lake_id)
+
+        s = select([func.ST_UnknownFunction(lake.geom, 2)])
+        with pytest.raises(ProgrammingError):
+            # TODO: here the query fails because of a
+            # "(psycopg2.ProgrammingError) can't adapt type 'WKBElement'"
+            # It would be better if it could fail because of a "UndefinedFunction" error
+            session.execute(s)
+
 
 class TestReflection():
 
@@ -573,8 +850,103 @@ class TestReflection():
             assert type_.geometry_type == 'LINESTRING'
             assert type_.srid == 4326
 
-    @postgis2_required
+    @skip_postgis1(postgis_version)
     def test_raster_reflection(self):
         t = Table('ocean', MetaData(), autoload=True, autoload_with=engine)
         type_ = t.c.rast.type
         assert isinstance(type_, Raster)
+
+
+class TestSTAsGeoJson():
+    InternalBase = declarative_base()
+
+    class TblWSpacesAndDots(InternalBase):
+        """
+        Dummy class to test names with dots and spaces.
+        No metadata is attached so the dialect is default SQL, not postgresql.
+        """
+        __tablename__ = "this is.an AWFUL.name"
+        __table_args__ = {'schema': 'another AWFUL.name for.schema'}
+
+        id = Column(Integer, primary_key=True)
+        geom = Column(String)
+
+    @staticmethod
+    def _assert_stmt(stmt, expected):
+        strstmt = str(stmt)
+        strstmt = strstmt.replace("\n", "")
+        assert strstmt == expected
+
+    def test_one(self):
+        stmt = select([func.ST_AsGeoJSON(Lake.__table__.c.geom)])
+
+        self._assert_stmt(
+            stmt, 'SELECT ST_AsGeoJSON(gis.lake.geom) AS "ST_AsGeoJSON_1" FROM gis.lake'
+        )
+
+    def test_two(self):
+        stmt = select([func.ST_AsGeoJSON(Lake, "geom")])
+        self._assert_stmt(
+            stmt,
+            'SELECT ST_AsGeoJSON(lake, %(ST_AsGeoJSON_2)s) AS '
+            '"ST_AsGeoJSON_1" FROM gis.lake',
+        )
+
+    @skip_postgis1(postgis_version)
+    @skip_postgis2(postgis_version)
+    def test_three(self):
+        sq = select([Lake, bindparam("dummy_val", 10).label("dummy_attr")]).alias()
+        stmt = select([func.ST_AsGeoJSON(sq, "geom")])
+        self._assert_stmt(
+            stmt,
+            'SELECT ST_AsGeoJSON(anon_1, %(ST_AsGeoJSON_2)s) AS "ST_AsGeoJSON_1" '
+            "FROM (SELECT gis.lake.id AS id, gis.lake.geom AS geom, %(dummy_val)s AS "
+            "dummy_attr FROM gis.lake) AS anon_1",
+        )
+
+    @skip_postgis1(postgis_version)
+    @skip_postgis2(postgis_version)
+    def test_four(self):
+        stmt = select([func.ST_AsGeoJSON(TestSTAsGeoJson.TblWSpacesAndDots, "geom")])
+        self._assert_stmt(
+            stmt,
+            'SELECT ST_AsGeoJSON("this is.an AWFUL.name", :ST_AsGeoJSON_2) '
+            'AS "ST_AsGeoJSON_1" FROM "another AWFUL.name for.schema".'
+            '"this is.an AWFUL.name"',
+        )
+
+    @skip_postgis1(postgis_version)
+    @skip_postgis2(postgis_version)
+    def test_five(self):
+        stmt = select([func.ST_AsGeoJSON(TestSTAsGeoJson.TblWSpacesAndDots, "geom", 3)])
+        self._assert_stmt(
+            stmt,
+            'SELECT ST_AsGeoJSON("this is.an AWFUL.name", '
+            ':ST_AsGeoJSON_2, :ST_AsGeoJSON_3) '
+            'AS "ST_AsGeoJSON_1" FROM "another AWFUL.name for.schema".'
+            '"this is.an AWFUL.name"',
+        )
+
+    @skip_postgis1(postgis_version)
+    def test_nested_funcs(self):
+        stmt = select([func.ST_AsGeoJSON(func.ST_MakeValid(func.ST_MakePoint(1, 2)))])
+        self._assert_stmt(
+            stmt,
+            'SELECT '
+            'ST_AsGeoJSON(ST_MakeValid('
+            'ST_MakePoint(:ST_MakePoint_1, :ST_MakePoint_2)'
+            ')) AS "ST_AsGeoJSON_1"',
+        )
+
+    @skip_postgis1(postgis_version)
+    def test_unknown_func(self):
+        stmt = select([
+            func.ST_AsGeoJSON(func.ST_UnknownFunction(func.ST_MakePoint(1, 2)))
+        ])
+        self._assert_stmt(
+            stmt,
+            'SELECT '
+            'ST_AsGeoJSON(ST_UnknownFunction('
+            'ST_MakePoint(:ST_MakePoint_1, :ST_MakePoint_2)'
+            ')) AS "ST_AsGeoJSON_1"',
+        )
