@@ -55,11 +55,39 @@ def test_to_shape_WKTElement():
 
 
 def test_from_shape():
+    # Standard case: POINT(1 2)
+    expected = WKBElement(str('0101000000000000000000f03f0000000000000040'))
     p = Point(1, 2)
     e = from_shape(p)
     assert isinstance(e, WKBElement)
     assert isinstance(e.data, buffer)
+    assert e == expected
 
     s = shapely.wkb.loads(bytes(e.data))
     assert isinstance(s, Point)
-    assert p.equals(p)
+    assert s.equals(p)
+
+    # Standard case with SRID: SRID=2145;POINT(1 2)
+    expected2 = WKBElement(str('0101000000000000000000f03f0000000000000040'), srid=2154)
+    p = Point(1, 2)
+    e2 = from_shape(p, srid=2154)
+    assert isinstance(e2, WKBElement)
+    assert isinstance(e2.data, buffer)
+    assert e2 == expected2
+
+    s2 = shapely.wkb.loads(bytes(e2.data))
+    assert isinstance(s2, Point)
+    assert s2.equals(p)
+
+    # Extended case: SRID=2145;POINT(1 2)
+    expected3 = WKBElement(
+        str('01010000206a080000000000000000f03f0000000000000040'),
+        extended=True)
+    e3 = from_shape(p, srid=2154, extended=True)
+    assert isinstance(e3, WKBElement)
+    assert isinstance(e3.data, buffer)
+    assert e3 == expected3
+
+    s3 = shapely.wkb.loads(bytes(e3.data))
+    assert isinstance(s, Point)
+    assert s3.equals(p)
